// Copyright (c) Microsoft Corporation
// All rights reserved. 
//
// Licensed under the Apache License, Version 2.0 (the ""License""); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
//
// THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//
// See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.

#define NTDDI_VERSION 0x06010000

#include <v8.h>
#include <string>
#include <node_object_wrap.h>
#include <ppltasks.h>
#include "CollectionsConverter.h"
#include "CollectionsWrap.h"
#include "node-async.h"
#include "NodeRtUtils.h"
#include "OpaqueWrapper.h"
#include "WrapperBase.h"

#using <Windows.WinMD>

// this undefs fixes the issues of compiling Windows.Data.Json, Windows.Storag.FileProperties, and Windows.Stroage.Search
// Some of the node header files brings windows definitions with the same names as some of the WinRT methods
#undef DocumentProperties
#undef GetObject
#undef CreateEvent
#undef FindText
#undef SendMessage

const char* REGISTRATION_TOKEN_MAP_PROPERTY_NAME = "__registrationTokenMap__";

using namespace v8;
using namespace node;
using namespace concurrency;

namespace NodeRT { namespace Windows { namespace Media { namespace Render { 


  static v8::Handle<v8::Value> InitAudioRenderCategoryEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("AudioRenderCategory"), enumObject);

    enumObject->Set(String::NewSymbol("other"), Integer::New(0));
    enumObject->Set(String::NewSymbol("foregroundOnlyMedia"), Integer::New(1));
    enumObject->Set(String::NewSymbol("backgroundCapableMedia"), Integer::New(2));
    enumObject->Set(String::NewSymbol("communications"), Integer::New(3));
    enumObject->Set(String::NewSymbol("alerts"), Integer::New(4));
    enumObject->Set(String::NewSymbol("soundEffects"), Integer::New(5));
    enumObject->Set(String::NewSymbol("gameEffects"), Integer::New(6));
    enumObject->Set(String::NewSymbol("gameMedia"), Integer::New(7));

    return scope.Close(Undefined());
  }



  
} } } } 

void init(Handle<Object> exports)
{
  if (FAILED(CoInitializeEx(nullptr, COINIT_MULTITHREADED)))
  {
    ThrowException(v8::Exception::Error(NodeRT::Utils::NewString(L"error in CoInitializeEx()")));
    return;
  }
  
  NodeRT::Windows::Media::Render::InitAudioRenderCategoryEnum(exports);

  NodeRT::Utils::RegisterNameSpace("Windows.Media.Render", exports);
}


NODE_MODULE(binding, init)